@@ -43,28 +43,33 @@
 
 #include "tv_device.h"
 
+#include "upnp.h"
+#include "upnpdebug.h"
+
 #include <assert.h>
 
+#include "posix_overwrites.h" // IWYU pragma: keep
+
 #define DEFAULT_WEB_DIR "./web"
 
 #define DESC_URL_SIZE 200
 
 /*! Global arrays for storing Tv Control Service variable names, values,
  * and defaults. */
-const char *tvc_varname[] = {"Power", "Channel", "Volume"};
+static const char *tvc_varname[] = {"Power", "Channel", "Volume"};
 
-char tvc_varval[TV_CONTROL_VARCOUNT][TV_MAX_VAL_LEN];
-const char *tvc_varval_def[] = {"1", "1", "5"};
+static char tvc_varval[TV_CONTROL_VARCOUNT][TV_MAX_VAL_LEN];
+static const char *tvc_varval_def[] = {"1", "1", "5"};
 
 /*! Global arrays for storing Tv Picture Service variable names, values,
  * and defaults. */
-const char *tvp_varname[] = {"Color", "Tint", "Contrast", "Brightness"};
+static const char *tvp_varname[] = {"Color", "Tint", "Contrast", "Brightness"};
 
-char tvp_varval[TV_PICTURE_VARCOUNT][TV_MAX_VAL_LEN];
-const char *tvp_varval_def[] = {"5", "5", "5", "5"};
+static char tvp_varval[TV_PICTURE_VARCOUNT][TV_MAX_VAL_LEN];
+static const char *tvp_varval_def[] = {"5", "5", "5", "5"};
 
 /*! The amount of time (in seconds) before advertisements will expire. */
-int default_advr_expire = 100;
+static int default_advr_expire = 100;
 
 /*! Global structure for storing the state table for this device. */
 struct TvService tv_service_table[2];
@@ -303,8 +308,8 @@ error_handler:
 int TvDeviceHandleSubscriptionRequest(const UpnpSubscriptionRequest *sr_event)
 {
 	unsigned int i = 0;
-	int cmp1 = 0;
-	int cmp2 = 0;
+	int cmp1;
+	int cmp2;
 	const char *l_serviceId = NULL;
 	const char *l_udn = NULL;
 	const char *l_sid = NULL;
@@ -359,7 +364,7 @@ int TvDeviceHandleSubscriptionRequest(const UpnpSubscriptionRequest *sr_event)
 int TvDeviceHandleGetVarRequest(UpnpStateVarRequest *cgv_event)
 {
 	unsigned int i = 0;
-	int j = 0;
+	int j;
 	int getvar_succeeded = 0;
 
 	UpnpStateVarRequest_set_CurrentVal(cgv_event, NULL);
@@ -672,7 +677,7 @@ int IncrementChannel(int incr,
 
 	ithread_mutex_lock(&TVDevMutex);
 	curchannel = atoi(tv_service_table[TV_SERVICE_CONTROL]
-				  .VariableStrVal[TV_CONTROL_CHANNEL]);
+			.VariableStrVal[TV_CONTROL_CHANNEL]);
 	ithread_mutex_unlock(&TVDevMutex);
 
 	newchannel = curchannel + incr;
@@ -718,41 +723,49 @@ int TvDeviceIncreaseChannel(
 int TvDeviceSetVolume(
 	IXML_Document *in, IXML_Document **out, const char **errorString)
 {
-	char *value = NULL;
-	int volume = 0;
+	int ret;
+	char *value;
+	int volume;
 
 	(*out) = NULL;
 	(*errorString) = NULL;
-	if (!(value = SampleUtil_GetFirstDocumentItem(in, "Volume"))) {
+
+	value = SampleUtil_GetFirstDocumentItem(in, "Volume");
+	if (!value) {
 		(*errorString) = "Invalid Volume";
-		return UPNP_E_INVALID_PARAM;
+		ret = UPNP_E_INVALID_PARAM;
+		goto end_function_1;
 	}
 	volume = atoi(value);
 	if (volume < MIN_VOLUME || volume > MAX_VOLUME) {
 		SampleUtil_Print("error: can't change to volume %d\n", volume);
 		(*errorString) = "Invalid Volume";
-		return UPNP_E_INVALID_PARAM;
+		ret = UPNP_E_INVALID_PARAM;
+		goto end_function_2;
 	}
 	/* Vendor-specific code to set the volume goes here. */
-	if (TvDeviceSetServiceTableVar(
+	if (!TvDeviceSetServiceTableVar(
 		    TV_SERVICE_CONTROL, TV_CONTROL_VOLUME, value)) {
-		if (UpnpAddToActionResponse(out,
-			    "SetVolume",
-			    TvServiceType[TV_SERVICE_CONTROL],
-			    "NewVolume",
-			    value) != UPNP_E_SUCCESS) {
-			(*out) = NULL;
-			(*errorString) = "Internal Error";
-			free(value);
-			return UPNP_E_INTERNAL_ERROR;
-		}
-		free(value);
-		return UPNP_E_SUCCESS;
-	} else {
-		free(value);
 		(*errorString) = "Internal Error";
-		return UPNP_E_INTERNAL_ERROR;
+		ret = UPNP_E_INTERNAL_ERROR;
+		goto end_function_2;
 	}
+	if (UpnpAddToActionResponse(out,
+		    "SetVolume",
+		    TvServiceType[TV_SERVICE_CONTROL],
+		    "NewVolume",
+		    value) != UPNP_E_SUCCESS) {
+		(*out) = NULL;
+		(*errorString) = "Internal Error";
+		ret = UPNP_E_INTERNAL_ERROR;
+		goto end_function_2;
+	}
+	ret = UPNP_E_SUCCESS;
+
+end_function_2:
+	free(value);
+end_function_1:
+	return ret;
 }
 
 /*!
@@ -783,7 +796,7 @@ static int IncrementVolume(
 
 	ithread_mutex_lock(&TVDevMutex);
 	curvolume = atoi(tv_service_table[TV_SERVICE_CONTROL]
-				 .VariableStrVal[TV_CONTROL_VOLUME]);
+			.VariableStrVal[TV_CONTROL_VOLUME]);
 	ithread_mutex_unlock(&TVDevMutex);
 
 	newvolume = curvolume + incr;
@@ -828,41 +841,49 @@ int TvDeviceDecreaseVolume(
 int TvDeviceSetColor(
 	IXML_Document *in, IXML_Document **out, const char **errorString)
 {
-	char *value = NULL;
-	int color = 0;
+	int ret;
+	char *value;
+	int color;
 
 	(*out) = NULL;
 	(*errorString) = NULL;
-	if (!(value = SampleUtil_GetFirstDocumentItem(in, "Color"))) {
+
+	value = SampleUtil_GetFirstDocumentItem(in, "Color");
+	if (!value) {
 		(*errorString) = "Invalid Color";
-		return UPNP_E_INVALID_PARAM;
+		ret = UPNP_E_INVALID_PARAM;
+		goto end_function_1;
 	}
 	color = atoi(value);
 	if (color < MIN_COLOR || color > MAX_COLOR) {
 		SampleUtil_Print("error: can't change to color %d\n", color);
 		(*errorString) = "Invalid Color";
-		return UPNP_E_INVALID_PARAM;
+		ret = UPNP_E_INVALID_PARAM;
+		goto end_function_2;
 	}
 	/* Vendor-specific code to set the volume goes here. */
-	if (TvDeviceSetServiceTableVar(
+	if (!TvDeviceSetServiceTableVar(
 		    TV_SERVICE_PICTURE, TV_PICTURE_COLOR, value)) {
-		if (UpnpAddToActionResponse(out,
-			    "SetColor",
-			    TvServiceType[TV_SERVICE_PICTURE],
-			    "NewColor",
-			    value) != UPNP_E_SUCCESS) {
-			(*out) = NULL;
-			(*errorString) = "Internal Error";
-			free(value);
-			return UPNP_E_INTERNAL_ERROR;
-		}
-		free(value);
-		return UPNP_E_SUCCESS;
-	} else {
-		free(value);
 		(*errorString) = "Internal Error";
-		return UPNP_E_INTERNAL_ERROR;
+		ret = UPNP_E_INTERNAL_ERROR;
+		goto end_function_2;
 	}
+	if (UpnpAddToActionResponse(out,
+		    "SetColor",
+		    TvServiceType[TV_SERVICE_PICTURE],
+		    "NewColor",
+		    value) != UPNP_E_SUCCESS) {
+		(*out) = NULL;
+		(*errorString) = "Internal Error";
+		ret = UPNP_E_INTERNAL_ERROR;
+		goto end_function_2;
+	}
+	ret = UPNP_E_SUCCESS;
+
+end_function_2:
+	free(value);
+end_function_1:
+	return ret;
 }
 
 /*!
@@ -893,7 +914,7 @@ static int IncrementColor(
 
 	ithread_mutex_lock(&TVDevMutex);
 	curcolor = atoi(tv_service_table[TV_SERVICE_PICTURE]
-				.VariableStrVal[TV_PICTURE_COLOR]);
+			.VariableStrVal[TV_PICTURE_COLOR]);
 	ithread_mutex_unlock(&TVDevMutex);
 
 	newcolor = curcolor + incr;
@@ -937,41 +958,49 @@ int TvDeviceIncreaseColor(
 int TvDeviceSetTint(
 	IXML_Document *in, IXML_Document **out, const char **errorString)
 {
-	char *value = NULL;
-	int tint = -1;
+	int ret;
+	char *value;
+	int tint;
 
 	(*out) = NULL;
 	(*errorString) = NULL;
-	if (!(value = SampleUtil_GetFirstDocumentItem(in, "Tint"))) {
+
+	value = SampleUtil_GetFirstDocumentItem(in, "Tint");
+	if (!value) {
 		(*errorString) = "Invalid Tint";
-		return UPNP_E_INVALID_PARAM;
+		ret = UPNP_E_INVALID_PARAM;
+		goto end_function_1;
 	}
 	tint = atoi(value);
 	if (tint < MIN_TINT || tint > MAX_TINT) {
 		SampleUtil_Print("error: can't change to tint %d\n", tint);
 		(*errorString) = "Invalid Tint";
-		return UPNP_E_INVALID_PARAM;
+		ret = UPNP_E_INVALID_PARAM;
+		goto end_function_2;
 	}
 	/* Vendor-specific code to set the volume goes here. */
 	if (TvDeviceSetServiceTableVar(
 		    TV_SERVICE_PICTURE, TV_PICTURE_TINT, value)) {
-		if (UpnpAddToActionResponse(out,
-			    "SetTint",
-			    TvServiceType[TV_SERVICE_PICTURE],
-			    "NewTint",
-			    value) != UPNP_E_SUCCESS) {
-			(*out) = NULL;
-			(*errorString) = "Internal Error";
-			free(value);
-			return UPNP_E_INTERNAL_ERROR;
-		}
-		free(value);
-		return UPNP_E_SUCCESS;
-	} else {
-		free(value);
 		(*errorString) = "Internal Error";
-		return UPNP_E_INTERNAL_ERROR;
+		ret = UPNP_E_INTERNAL_ERROR;
+		goto end_function_2;
 	}
+	if (UpnpAddToActionResponse(out,
+		    "SetTint",
+		    TvServiceType[TV_SERVICE_PICTURE],
+		    "NewTint",
+		    value) != UPNP_E_SUCCESS) {
+		(*out) = NULL;
+		(*errorString) = "Internal Error";
+		ret = UPNP_E_INTERNAL_ERROR;
+		goto end_function_2;
+	}
+	ret = UPNP_E_SUCCESS;
+
+end_function_2:
+	free(value);
+end_function_1:
+	return ret;
 }
 
 /******************************************************************************
@@ -1007,7 +1036,7 @@ int IncrementTint(int incr,
 
 	ithread_mutex_lock(&TVDevMutex);
 	curtint = atoi(tv_service_table[TV_SERVICE_PICTURE]
-			       .VariableStrVal[TV_PICTURE_TINT]);
+			.VariableStrVal[TV_PICTURE_TINT]);
 	ithread_mutex_unlock(&TVDevMutex);
 
 	newtint = curtint + incr;
@@ -1092,43 +1121,50 @@ int TvDeviceDecreaseTint(
 int TvDeviceSetContrast(
 	IXML_Document *in, IXML_Document **out, const char **errorString)
 {
-	char *value = NULL;
-	int contrast = -1;
+	int ret;
+	char *value;
+	int contrast;
 
 	(*out) = NULL;
 	(*errorString) = NULL;
 
-	if (!(value = SampleUtil_GetFirstDocumentItem(in, "Contrast"))) {
+	value = SampleUtil_GetFirstDocumentItem(in, "Contrast");
+	if (!value) {
 		(*errorString) = "Invalid Contrast";
-		return UPNP_E_INVALID_PARAM;
+		ret = UPNP_E_INVALID_PARAM;
+		goto end_function_1;
 	}
 	contrast = atoi(value);
 	if (contrast < MIN_CONTRAST || contrast > MAX_CONTRAST) {
 		SampleUtil_Print(
 			"error: can't change to contrast %d\n", contrast);
 		(*errorString) = "Invalid Contrast";
-		return UPNP_E_INVALID_PARAM;
+		ret = UPNP_E_INVALID_PARAM;
+		goto end_function_2;
 	}
 	/* Vendor-specific code to set the volume goes here. */
-	if (TvDeviceSetServiceTableVar(
+	if (!TvDeviceSetServiceTableVar(
 		    TV_SERVICE_PICTURE, TV_PICTURE_CONTRAST, value)) {
-		if (UpnpAddToActionResponse(out,
-			    "SetContrast",
-			    TvServiceType[TV_SERVICE_PICTURE],
-			    "NewContrast",
-			    value) != UPNP_E_SUCCESS) {
-			(*out) = NULL;
-			(*errorString) = "Internal Error";
-			free(value);
-			return UPNP_E_INTERNAL_ERROR;
-		}
-		free(value);
-		return UPNP_E_SUCCESS;
-	} else {
-		free(value);
 		(*errorString) = "Internal Error";
-		return UPNP_E_INTERNAL_ERROR;
+		ret = UPNP_E_INTERNAL_ERROR;
+		goto end_function_2;
 	}
+	if (UpnpAddToActionResponse(out,
+		    "SetContrast",
+		    TvServiceType[TV_SERVICE_PICTURE],
+		    "NewContrast",
+		    value) != UPNP_E_SUCCESS) {
+		(*out) = NULL;
+		(*errorString) = "Internal Error";
+		ret = UPNP_E_INTERNAL_ERROR;
+		goto end_function_2;
+	}
+	ret = UPNP_E_SUCCESS;
+
+end_function_2:
+	free(value);
+end_function_1:
+	return ret;
 }
 
 /*!
@@ -1159,7 +1195,7 @@ static int IncrementContrast(
 
 	ithread_mutex_lock(&TVDevMutex);
 	curcontrast = atoi(tv_service_table[TV_SERVICE_PICTURE]
-				   .VariableStrVal[TV_PICTURE_CONTRAST]);
+			.VariableStrVal[TV_PICTURE_CONTRAST]);
 	ithread_mutex_unlock(&TVDevMutex);
 
 	newcontrast = curcontrast + incr;
@@ -1204,42 +1240,50 @@ int TvDeviceDecreaseContrast(
 int TvDeviceSetBrightness(
 	IXML_Document *in, IXML_Document **out, const char **errorString)
 {
-	char *value = NULL;
-	int brightness = -1;
+	int ret;
+	char *value;
+	int brightness;
 
 	(*out) = NULL;
 	(*errorString) = NULL;
-	if (!(value = SampleUtil_GetFirstDocumentItem(in, "Brightness"))) {
+
+	value = SampleUtil_GetFirstDocumentItem(in, "Brightness");
+	if (!value) {
 		(*errorString) = "Invalid Brightness";
-		return UPNP_E_INVALID_PARAM;
+		ret = UPNP_E_INVALID_PARAM;
+		goto end_function_1;
 	}
 	brightness = atoi(value);
 	if (brightness < MIN_BRIGHTNESS || brightness > MAX_BRIGHTNESS) {
 		SampleUtil_Print(
 			"error: can't change to brightness %d\n", brightness);
 		(*errorString) = "Invalid Brightness";
-		return UPNP_E_INVALID_PARAM;
+		ret = UPNP_E_INVALID_PARAM;
+		goto end_function_2;
 	}
 	/* Vendor-specific code to set the volume goes here. */
-	if (TvDeviceSetServiceTableVar(
+	if (!TvDeviceSetServiceTableVar(
 		    TV_SERVICE_PICTURE, TV_PICTURE_BRIGHTNESS, value)) {
-		if (UpnpAddToActionResponse(out,
-			    "SetBrightness",
-			    TvServiceType[TV_SERVICE_PICTURE],
-			    "NewBrightness",
-			    value) != UPNP_E_SUCCESS) {
-			(*out) = NULL;
-			(*errorString) = "Internal Error";
-			free(value);
-			return UPNP_E_INTERNAL_ERROR;
-		}
-		free(value);
-		return UPNP_E_SUCCESS;
-	} else {
-		free(value);
 		(*errorString) = "Internal Error";
-		return UPNP_E_INTERNAL_ERROR;
+		ret = UPNP_E_INTERNAL_ERROR;
+		goto end_function_2;
 	}
+	if (UpnpAddToActionResponse(out,
+		    "SetBrightness",
+		    TvServiceType[TV_SERVICE_PICTURE],
+		    "NewBrightness",
+		    value) != UPNP_E_SUCCESS) {
+		(*out) = NULL;
+		(*errorString) = "Internal Error";
+		ret = UPNP_E_INTERNAL_ERROR;
+		goto end_function_2;
+	}
+	ret = UPNP_E_SUCCESS;
+
+end_function_2:
+	free(value);
+end_function_1:
+	return ret;
 }
 
 /*!
@@ -1270,7 +1314,7 @@ static int IncrementBrightness(
 
 	ithread_mutex_lock(&TVDevMutex);
 	curbrightness = atoi(tv_service_table[TV_SERVICE_PICTURE]
-				     .VariableStrVal[TV_PICTURE_BRIGHTNESS]);
+			.VariableStrVal[TV_PICTURE_BRIGHTNESS]);
 	ithread_mutex_unlock(&TVDevMutex);
 
 	newbrightness = curbrightness + incr;
@@ -1313,13 +1357,13 @@ int TvDeviceDecreaseBrightness(
 }
 
 int TvDeviceCallbackEventHandler(
-	Upnp_EventType EventType, const void *Event, void *Cookie)
+	Upnp_EventType EventType, void *Event, void *Cookie)
 {
 	(void)Cookie;
 	switch (EventType) {
 	case UPNP_EVENT_SUBSCRIPTION_REQUEST:
 		TvDeviceHandleSubscriptionRequest(
-			(UpnpSubscriptionRequest *)Event);
+			(const UpnpSubscriptionRequest *)Event);
 		break;
 	case UPNP_CONTROL_GET_VAR_REQUEST:
 		TvDeviceHandleGetVarRequest((UpnpStateVarRequest *)Event);
@@ -1341,7 +1385,7 @@ int TvDeviceCallbackEventHandler(
 		break;
 	default:
 		SampleUtil_Print("Error in TvDeviceCallbackEventHandler: "
-				 "unknown event type %d\n",
+				 "unknown event type %u\n",
 			EventType);
 	}
 	/* Print a summary of the event received */
@@ -1350,32 +1394,55 @@ int TvDeviceCallbackEventHandler(
 	return 0;
 }
 
-int TvDeviceStart(char *ip_address,
+int TvDeviceStart(char *iface,
 	unsigned short port,
 	const char *desc_doc_name,
 	const char *web_dir_path,
+	int ip_mode,
 	print_string pfun,
 	int combo)
 {
 	int ret = UPNP_E_SUCCESS;
 	char desc_doc_url[DESC_URL_SIZE];
+	char *ip_address = NULL;
+	int address_family = AF_INET;
 
 	ithread_mutex_init(&TVDevMutex, NULL);
-
+	UpnpSetLogFileNames(NULL, NULL);
+	UpnpSetLogLevel(UPNP_ALL);
+	UpnpInitLog();
 	SampleUtil_Initialize(pfun);
 	SampleUtil_Print("Initializing UPnP Sdk with\n"
-			 "\tipaddress = %s port = %u\n",
-		ip_address ? ip_address : "{NULL}",
+			 "\tinterface = %s port = %u\n",
+		iface ? iface : "{NULL}",
 		port);
-	ret = UpnpInit2(ip_address, port);
+	ret = UpnpInit2(iface, port);
 	if (ret != UPNP_E_SUCCESS) {
 		SampleUtil_Print("Error with UpnpInit2 -- %d\n", ret);
 		UpnpFinish();
 
 		return ret;
 	}
-	ip_address = UpnpGetServerIpAddress();
-	port = UpnpGetServerPort();
+	switch (ip_mode) {
+	case IP_MODE_IPV4:
+		ip_address = UpnpGetServerIpAddress();
+		port = UpnpGetServerPort();
+		address_family = AF_INET;
+		break;
+	case IP_MODE_IPV6_LLA:
+		ip_address = UpnpGetServerIp6Address();
+		port = UpnpGetServerPort6();
+		address_family = AF_INET6;
+		break;
+	case IP_MODE_IPV6_ULA_GUA:
+		ip_address = UpnpGetServerUlaGuaIp6Address();
+		port = UpnpGetServerUlaGuaPort6();
+		address_family = AF_INET6;
+		break;
+	default:
+		SampleUtil_Print("Invalid ip_mode : %d\n", ip_mode);
+		return UPNP_E_INTERNAL_ERROR;
+	}
 	SampleUtil_Print("UPnP Initialized\n"
 			 "\tipaddress = %s port = %u\n",
 		ip_address ? ip_address : "{NULL}",
@@ -1390,12 +1457,26 @@ int TvDeviceStart(char *ip_address,
 	if (!web_dir_path) {
 		web_dir_path = DEFAULT_WEB_DIR;
 	}
-	snprintf(desc_doc_url,
-		DESC_URL_SIZE,
-		"http://%s:%d/%s",
-		ip_address,
-		port,
-		desc_doc_name);
+	switch (address_family) {
+	case AF_INET:
+		snprintf(desc_doc_url,
+			DESC_URL_SIZE,
+			"http://%s:%d/%s",
+			ip_address,
+			port,
+			desc_doc_name);
+		break;
+	case AF_INET6:
+		snprintf(desc_doc_url,
+			DESC_URL_SIZE,
+			"http://[%s]:%d/%s",
+			ip_address,
+			port,
+			desc_doc_name);
+		break;
+	default:
+		return UPNP_E_INTERNAL_ERROR;
+	}
 	SampleUtil_Print("Specifying the webserver root directory -- %s\n",
 		web_dir_path);
 	ret = UpnpSetWebServerRootDir(web_dir_path);
@@ -1411,10 +1492,11 @@ int TvDeviceStart(char *ip_address,
 	SampleUtil_Print("Registering the RootDevice\n"
 			 "\t with desc_doc_url: %s\n",
 		desc_doc_url);
-	ret = UpnpRegisterRootDevice(desc_doc_url,
+	ret = UpnpRegisterRootDevice3(desc_doc_url,
 		TvDeviceCallbackEventHandler,
 		&device_handle,
-		&device_handle);
+		&device_handle,
+		address_family);
 	if (ret != UPNP_E_SUCCESS) {
 		SampleUtil_Print(
 			"Error registering the rootdevice : %d\n", ret);
@@ -1452,13 +1534,12 @@ int TvDeviceStop(void)
 
 void *TvDeviceCommandLoop(void *args)
 {
-	int stoploop = 0;
 	char cmdline[100];
 	char cmd[100];
 	char *s;
 	(void)args;
 
-	while (!stoploop) {
+	while (1) {
 		sprintf(cmdline, " ");
 		sprintf(cmd, " ");
 		SampleUtil_Print("\n>> ");
@@ -1466,7 +1547,11 @@ void *TvDeviceCommandLoop(void *args)
 		s = fgets(cmdline, 100, stdin);
 		if (!s)
 			break;
+#ifdef _WIN32
+		sscanf_s(cmdline, "%s", cmd, (unsigned)_countof(cmd));
+#else
 		sscanf(cmdline, "%s", cmd);
+#endif
 		if (strcasecmp(cmd, "exit") == 0) {
 			SampleUtil_Print("Shutting down...\n");
 			TvDeviceStop();
@@ -1484,33 +1569,45 @@ void *TvDeviceCommandLoop(void *args)
 int device_main(int argc, char *argv[])
 {
 	unsigned int portTemp = 0;
-	char *ip_address = NULL;
+	char *iface = NULL;
 	char *desc_doc_name = NULL;
 	char *web_dir_path = NULL;
 	unsigned short port = 0;
+	int ip_mode = IP_MODE_IPV4;
 	int i = 0;
 
 	SampleUtil_Initialize(linux_print);
 	/* Parse options */
 	for (i = 1; i < argc; i++) {
-		if (strcmp(argv[i], "-ip") == 0) {
-			ip_address = argv[++i];
+		if (strcmp(argv[i], "-i") == 0) {
+			iface = argv[++i];
 		} else if (strcmp(argv[i], "-port") == 0) {
+#ifdef WIN32
+			sscanf_s(argv[++i], "%u", &portTemp);
+#else
 			sscanf(argv[++i], "%u", &portTemp);
+#endif
 		} else if (strcmp(argv[i], "-desc") == 0) {
 			desc_doc_name = argv[++i];
 		} else if (strcmp(argv[i], "-webdir") == 0) {
 			web_dir_path = argv[++i];
+		} else if (strcmp(argv[i], "-m") == 0) {
+#ifdef _WIN32
+			sscanf_s(argv[++i], "%d", &ip_mode);
+#else
+			sscanf(argv[++i], "%d", &ip_mode);
+#endif
 		} else if (strcmp(argv[i], "-help") == 0) {
 			SampleUtil_Print(
-				"Usage: %s -ip ipaddress -port port"
+				"Usage: %s -i interface -port port"
 				" -desc desc_doc_name -webdir web_dir_path"
-				" -help (this message)\n",
+				" -m ip_mode -help (this message)\n",
 				argv[0]);
 			SampleUtil_Print(
-				"\tipaddress:     IP address of the device"
+				"\tinterface:     interface address of the "
+				"device"
 				" (must match desc. doc)\n"
-				"\t\te.g.: 192.168.0.4\n"
+				"\t\te.g.: eth0\n"
 				"\tport:          Port number to use for"
 				" receiving UPnP messages (must match desc. "
 				"doc)\n"
@@ -1518,16 +1615,23 @@ int device_main(int argc, char *argv[])
 				"\tdesc_doc_name: name of device description "
 				"document\n"
 				"\t\te.g.: tvdevicedesc.xml\n"
-				"\tweb_dir_path: Filesystem path where web "
+				"\tweb_dir_path:  Filesystem path where web "
 				"files"
 				" related to the device are stored\n"
-				"\t\te.g.: /upnp/sample/tvdevice/web\n");
+				"\t\te.g.: /upnp/sample/tvdevice/web\n"
+				"\tip_mode:       set to 1 for IPv4 (default), "
+				"2 for IPv6 LLA and 3 for IPv6 ULA or GUA\n");
 			return 1;
 		}
 	}
 	port = (unsigned short)portTemp;
-	return TvDeviceStart(
-		ip_address, port, desc_doc_name, web_dir_path, linux_print, 0);
+	return TvDeviceStart(iface,
+		port,
+		desc_doc_name,
+		web_dir_path,
+		ip_mode,
+		linux_print,
+		0);
 }
 
 /*! @} Device Sample Module */
