@@ -54,7 +54,7 @@
 	#include "ithread.h"
 	#include "ssdplib.h"
 	#include "statcodes.h"
-	#include "unixutil.h" /* for socklen_t, EAFNOSUPPORT */
+	#include "unixutil.h" /* for socklen_t, EAFNOSUPPORT */ // IWYU pragma: keep
 	#include "upnpapi.h"
 	#include "upnputil.h"
 
@@ -88,12 +88,25 @@ typedef enum
 } MiniServerState;
 
 /*! . */
-uint16_t miniStopSockPort;
+static uint16_t miniStopSockPort;
 
 /*!
  * module vars
  */
 static MiniServerState gMServState = MSERV_IDLE;
+
+/*! Global list to track active socket connections */
+static LinkedList gActiveConnections;
+static ithread_mutex_t gActiveConnectionsMutex;
+static int gActiveConnectionsInitialized = 0;
+
+/*! Structure to hold active connection info */
+struct active_connection_t
+{
+	SOCKET socket;
+	time_t connect_time;
+};
+
 	#ifdef INTERNAL_WEB_SERVER
 static MiniServerCallback gGetCallback = NULL;
 static MiniServerCallback gSoapCallback = NULL;
@@ -238,7 +251,7 @@ static int dispatch_request(
 	http_message_t *request;
 	MiniServerCallback callback;
 	WebCallback_HostValidate host_validate_callback = 0;
-	void *cookie;
+	void *cookie = 0;
 	int rc = UPNP_E_SUCCESS;
 	/* If it does not fit in here, it is likely invalid anyway. */
 	char host_port[NAME_SIZE];
@@ -325,16 +338,17 @@ static int dispatch_request(
 			goto ExitFunction;
 		} else {
 			membuffer redir_buf;
-			static const char *redir_fmt =
-				"HTTP/1.1 307 Temporary Redirect\r\n"
-				"Location: http://%s\r\n\r\n";
-			char redir_str[NAME_SIZE];
+			char redir_str[2 * NAME_SIZE];
 			int timeout = HTTP_DEFAULT_TIMEOUT;
 
 			getNumericHostRedirection(
 				(int)info->socket, host_port, sizeof host_port);
 			membuffer_init(&redir_buf);
-			snprintf(redir_str, NAME_SIZE, redir_fmt, host_port);
+			snprintf(redir_str,
+				sizeof redir_str,
+				"HTTP/1.1 307 Temporary Redirect\r\n"
+				"Location: http://%s\r\n\r\n",
+				host_port);
 			membuffer_append_str(&redir_buf, redir_str);
 			rc = http_SendMessage(info,
 				&timeout,
@@ -367,6 +381,130 @@ static UPNP_INLINE void handle_error(
 	http_SendStatusResponse(info, http_error_code, major, minor);
 }
 
+/*!
+ * \brief Compare function for active connections, return true if equal (found)
+ */
+static int active_connection_cmp(void *first, void *second)
+{
+	struct active_connection_t *conn1 = (struct active_connection_t *)first;
+	struct active_connection_t *conn2 =
+		(struct active_connection_t *)second;
+
+	/* Return non-zero (true) if sockets match */
+	return (conn1->socket == conn2->socket);
+}
+
+/*!
+ * \brief Add a socket to the active connections list
+ */
+static void add_active_connection(SOCKET sock)
+{
+	struct active_connection_t *conn;
+
+	if (!gActiveConnectionsInitialized) {
+		ListInit(&gActiveConnections, active_connection_cmp, free);
+		ithread_mutex_init(&gActiveConnectionsMutex, NULL);
+		gActiveConnectionsInitialized = 1;
+	}
+
+	conn = (struct active_connection_t *)malloc(
+		sizeof(struct active_connection_t));
+	if (conn) {
+		conn->socket = sock;
+		conn->connect_time = time(NULL);
+
+		ithread_mutex_lock(&gActiveConnectionsMutex);
+		ListAddTail(&gActiveConnections, conn);
+		ithread_mutex_unlock(&gActiveConnectionsMutex);
+
+		UpnpPrintf(UPNP_INFO,
+			MSERV,
+			__FILE__,
+			__LINE__,
+			"Added active connection: socket %d\n",
+			sock);
+	}
+}
+
+/*!
+ * \brief Remove a socket from the active connections list
+ */
+static void remove_active_connection(SOCKET sock)
+{
+	struct active_connection_t search_conn;
+	ListNode *node;
+
+	if (!gActiveConnectionsInitialized) {
+		return;
+	}
+	/* Create a temporary connection structure for searching */
+	search_conn.socket = sock;
+	search_conn.connect_time = 0; /* Not used for comparison */
+
+	ithread_mutex_lock(&gActiveConnectionsMutex);
+	node = ListFind(&gActiveConnections, NULL, &search_conn);
+	if (node) {
+		ListDelNode(&gActiveConnections, node, 1);
+		UpnpPrintf(UPNP_INFO,
+			MSERV,
+			__FILE__,
+			__LINE__,
+			"Removed active connection: socket %d\n",
+			sock);
+	}
+	ithread_mutex_unlock(&gActiveConnectionsMutex);
+}
+
+/*!
+ * \brief Shutdown all active socket connections
+ */
+void shutdown_all_active_connections(void)
+{
+	ListNode *node;
+	int count;
+
+	if (!gActiveConnectionsInitialized) {
+		return;
+	}
+	UpnpPrintf(UPNP_INFO,
+		MSERV,
+		__FILE__,
+		__LINE__,
+		"Shutting down all active socket connections\n");
+
+	ithread_mutex_lock(&gActiveConnectionsMutex);
+	node = ListHead(&gActiveConnections);
+	count = 0;
+	while (node) {
+		struct active_connection_t *conn =
+			(struct active_connection_t *)node->item;
+		if (conn && conn->socket != INVALID_SOCKET) {
+			UpnpPrintf(UPNP_INFO,
+				MSERV,
+				__FILE__,
+				__LINE__,
+				"Force shutting down socket %d\n",
+				conn->socket);
+			shutdown(conn->socket, SD_BOTH);
+			count++;
+		}
+		node = ListNext(&gActiveConnections, node);
+	}
+
+	/* Destroy the list and reset state */
+	ListDestroy(&gActiveConnections, 1);
+	gActiveConnectionsInitialized = 0;
+	ithread_mutex_unlock(&gActiveConnectionsMutex);
+	ithread_mutex_destroy(&gActiveConnectionsMutex);
+
+	UpnpPrintf(UPNP_INFO,
+		MSERV,
+		__FILE__,
+		__LINE__,
+		"Shutdown %d active socket connections and clean\n",
+		count);
+}
+
 /*!
  * \brief Free memory assigned for handling request and unitialize socket
  * functionality.
@@ -377,6 +515,7 @@ static void free_handle_request_arg(
 {
 	struct mserv_request_t *request = (struct mserv_request_t *)args;
 
+	remove_active_connection(request->connfd);
 	sock_close(request->connfd);
 	free(request);
 }
@@ -470,7 +609,7 @@ static UPNP_INLINE void schedule_request_job(
 
 	request = (struct mserv_request_t *)malloc(
 		sizeof(struct mserv_request_t));
-	if (request == NULL) {
+	if (!request) {
 		UpnpPrintf(UPNP_INFO,
 			MSERV,
 			__FILE__,
@@ -488,6 +627,10 @@ static UPNP_INLINE void schedule_request_job(
 	TPJobInit(&job, (start_routine)handle_request, (void *)request);
 	TPJobSetFreeFunction(&job, free_handle_request_arg);
 	TPJobSetPriority(&job, MED_PRIORITY);
+
+	/* Add the connection to active connections list */
+	add_active_connection(connfd);
+
 	if (ThreadPoolAdd(&gMiniServerThreadPool, &job, NULL) != 0) {
 		UpnpPrintf(UPNP_INFO,
 			MSERV,
@@ -495,6 +638,7 @@ static UPNP_INLINE void schedule_request_job(
 			__LINE__,
 			"mserv %d: cannot schedule request\n",
 			connfd);
+		remove_active_connection(connfd);
 		free(request);
 		sock_close(connfd);
 		return;
@@ -537,10 +681,21 @@ static void web_server_accept(SOCKET lsock, fd_set *set)
 	#endif /* INTERNAL_WEB_SERVER */
 }
 
-static void ssdp_read(SOCKET rsock, fd_set *set)
+static void ssdp_read(SOCKET *rsock, fd_set *set)
 {
-	if (rsock != INVALID_SOCKET && FD_ISSET(rsock, set)) {
-		readFromSSDPSocket(rsock);
+	if (*rsock != INVALID_SOCKET && FD_ISSET(*rsock, set)) {
+		int ret = readFromSSDPSocket(*rsock);
+		if (ret != 0) {
+			UpnpPrintf(UPNP_INFO,
+				MSERV,
+				__FILE__,
+				__LINE__,
+				"miniserver: Error in readFromSSDPSocket(%d): "
+				"closing socket\n",
+				*rsock);
+			sock_close(*rsock);
+			*rsock = INVALID_SOCKET;
+		}
 	}
 }
 
@@ -583,6 +738,13 @@ static int receive_from_stopSock(SOCKET ssock, fd_set *set)
 			if (NULL != strstr(requestBuf, "ShutDown")) {
 				return 1;
 			}
+		} else {
+			UpnpPrintf(UPNP_INFO,
+				MSERV,
+				__FILE__,
+				__LINE__,
+				"miniserver: stopSock Error, aborting...\n");
+			return 1;
 		}
 	}
 
@@ -658,16 +820,20 @@ static void RunMiniServer(
 			web_server_accept(
 				miniSock->miniServerSock6UlaGua, &rdSet);
 	#ifdef INCLUDE_CLIENT_APIS
-			ssdp_read(miniSock->ssdpReqSock4, &rdSet);
-			ssdp_read(miniSock->ssdpReqSock6, &rdSet);
+			ssdp_read(&miniSock->ssdpReqSock4, &rdSet);
+			ssdp_read(&miniSock->ssdpReqSock6, &rdSet);
 	#endif /* INCLUDE_CLIENT_APIS */
-			ssdp_read(miniSock->ssdpSock4, &rdSet);
-			ssdp_read(miniSock->ssdpSock6, &rdSet);
-			ssdp_read(miniSock->ssdpSock6UlaGua, &rdSet);
+			ssdp_read(&miniSock->ssdpSock4, &rdSet);
+			ssdp_read(&miniSock->ssdpSock6, &rdSet);
+			ssdp_read(&miniSock->ssdpSock6UlaGua, &rdSet);
 			stopSock = receive_from_stopSock(
 				miniSock->miniServerStopSock, &rdSet);
 		}
 	}
+
+	/* shutdown connections */
+	shutdown_all_active_connections();
+
 	/* Close all sockets. */
 	sock_close(miniSock->miniServerSock4);
 	sock_close(miniSock->miniServerSock6);
@@ -762,7 +928,6 @@ static int init_socket_suff(
 			"init_socket_suff(): Invalid IP version: %d.\n",
 			ip_version);
 		goto error;
-		break;
 	}
 
 	if (inet_pton(domain, text_addr, addr) <= 0)
@@ -1122,11 +1287,7 @@ static int get_miniserver_stopsock(
 	/* Bind to local socket. */
 	memset(&stop_sockaddr, 0, sizeof(stop_sockaddr));
 	stop_sockaddr.sin_family = (sa_family_t)AF_INET;
-	#ifdef _WIN32
 	inet_pton(AF_INET, "127.0.0.1", &stop_sockaddr.sin_addr);
-	#else
-	stop_sockaddr.sin_addr.s_addr = inet_addr("127.0.0.1");
-	#endif
 	ret = bind(miniServerStopSock,
 		(struct sockaddr *)&stop_sockaddr,
 		sizeof(stop_sockaddr));
@@ -1279,7 +1440,7 @@ int StartMiniServer(
 	return UPNP_E_SUCCESS;
 }
 
-int StopMiniServer()
+int StopMiniServer(void)
 {
 	char errorBuffer[ERROR_BUFFER_LEN];
 	socklen_t socklen = sizeof(struct sockaddr_in);
@@ -1308,15 +1469,11 @@ int StopMiniServer()
 	}
 	while (gMServState != (MiniServerState)MSERV_IDLE) {
 		ssdpAddr.sin_family = (sa_family_t)AF_INET;
-	#ifdef _WIN32
 		inet_pton(AF_INET, "127.0.0.1", &ssdpAddr.sin_addr);
-	#else
-		ssdpAddr.sin_addr.s_addr = inet_addr("127.0.0.1");
-	#endif
 		ssdpAddr.sin_port = htons(miniStopSockPort);
 		sendto(sock,
 			buf,
-			(int)bufLen,
+			bufLen,
 			0,
 			(struct sockaddr *)&ssdpAddr,
 			socklen);
