@@ -43,8 +43,8 @@
 
 	#include "webserver.h"
 
-	#include "ExtraHeaders.h"
-	#include "FileInfo.h"
+	#include "UpnpExtraHeaders.h"
+	#include "UpnpFileInfo.h"
 	#include "UpnpIntTypes.h"
 	#include "UpnpStdInt.h"
 	#include "VirtualDir.h"
@@ -64,9 +64,7 @@
 	#include <fcntl.h>
 	#include <sys/stat.h>
 
-	#ifdef _WIN32
-		#define snprintf _snprintf
-	#endif
+	#include "posix_overwrites.h" // IWYU pragma: keep
 
 /*!
  * Response Types.
@@ -145,6 +143,15 @@ static char *web_server_asctime_r(const struct tm *tm, char *buf)
 
 	asctime_s(buf, ASCTIME_R_BUFFER_SIZE, tm);
 	return buf;
+}
+	#elif defined(__sun__)
+static char *web_server_asctime_r(const struct tm *tm, char *buf)
+{
+	if (tm == NULL || buf == NULL)
+		return NULL;
+
+	asctime_r(tm, buf, ASCTIME_R_BUFFER_SIZE);
+	return buf;
 }
 	#else
 		#define web_server_asctime_r asctime_r
@@ -233,6 +240,8 @@ static struct document_type_t gMediaTypeList[NUM_MEDIA_TYPES];
 
 /*! Global variable. A local dir which serves as webserver root. */
 membuffer gDocumentRootDir;
+/*! Global variable. A string which is set in the header field. */
+membuffer gWebserverCorsString;
 
 /*! XML document. */
 static struct xml_alias_t gAliasDoc;
@@ -477,6 +486,7 @@ int web_server_init()
 		/* decode media list */
 		media_list_init();
 		membuffer_init(&gDocumentRootDir);
+		membuffer_init(&gWebserverCorsString);
 		glob_alias_init();
 		pVirtualDirList = NULL;
 
@@ -501,6 +511,7 @@ void web_server_destroy(void)
 {
 	if (bWebServerState == WEB_SERVER_ENABLED) {
 		membuffer_destroy(&gDocumentRootDir);
+		membuffer_destroy(&gWebserverCorsString);
 		alias_release(&gAliasDoc);
 
 		ithread_mutex_lock(&gWebMutex);
@@ -530,26 +541,44 @@ static int get_file_info(
 	int code;
 	struct stat s;
 	FILE *fp;
+	int fd;
 	int rc = 0;
 	time_t aux_LastModified;
 	struct tm date;
 	char buffer[ASCTIME_R_BUFFER_SIZE];
 
 	UpnpFileInfo_set_ContentType(info, NULL);
-	code = stat(filename, &s);
-	if (code == -1)
-		return -1;
-	if (S_ISDIR(s.st_mode))
-		UpnpFileInfo_set_IsDirectory(info, 1);
-	else if (S_ISREG(s.st_mode))
-		UpnpFileInfo_set_IsDirectory(info, 0);
-	else
-		return -1;
-	/* check readable */
+	#ifdef _WIN32
+	fopen_s(&fp, filename, "r");
+	#else
 	fp = fopen(filename, "r");
+	#endif
+	/* check readable */
 	UpnpFileInfo_set_IsReadable(info, fp != NULL);
-	if (fp)
-		fclose(fp);
+	if (!fp) {
+		rc = -1;
+		goto exit_function;
+	}
+	fd = fileno(fp);
+	if (fd == -1) {
+		rc = -1;
+		goto exit_function;
+	}
+	code = fstat(fd, &s);
+	if (code == -1) {
+		rc = -1;
+		goto exit_function;
+	}
+	fclose(fp);
+	fp = NULL;
+	if (S_ISDIR(s.st_mode)) {
+		UpnpFileInfo_set_IsDirectory(info, 1);
+	} else if (S_ISREG(s.st_mode)) {
+		UpnpFileInfo_set_IsDirectory(info, 0);
+	} else {
+		rc = -1;
+		goto exit_function;
+	}
 	UpnpFileInfo_set_FileLength(info, s.st_size);
 	UpnpFileInfo_set_LastModified(info, s.st_mtime);
 	rc = get_content_type(filename, info);
@@ -558,13 +587,17 @@ static int get_file_info(
 		HTTP,
 		__FILE__,
 		__LINE__,
-		"file info: %s, length: %lld, last_mod=%s readable=%d\n",
+		"file info: %s, length: %" PRId64 ", last_mod=%s readable=%d\n",
 		filename,
-		(long long)UpnpFileInfo_get_FileLength(info),
+		(int64_t)UpnpFileInfo_get_FileLength(info),
 		web_server_asctime_r(
 			http_gmtime_r(&aux_LastModified, &date), buffer),
 		UpnpFileInfo_get_IsReadable(info));
 
+exit_function:
+	if (fp) {
+		fclose(fp);
+	}
 	return rc;
 }
 
@@ -586,6 +619,17 @@ int web_server_set_root_dir(const char *root_dir)
 	return 0;
 }
 
+int web_server_set_cors(const char *cors_string)
+{
+	int ret;
+
+	ret = membuffer_assign_str(&gWebserverCorsString, cors_string);
+	if (ret != 0)
+		return ret;
+
+	return 0;
+}
+
 /*!
  * \brief Compare the files names between the one on the XML alias the one
  * passed in as the input parameter. If equal extract file information.
@@ -768,7 +812,11 @@ static int GetNextRange(
 	if ((Ptr = strstr(Tok, "-")) == NULL)
 		return -1;
 	*Ptr = ' ';
+	#ifdef _WIN32
+	sscanf_s(Tok, "%" SCNd64 "%" SCNd64, &F, &L);
+	#else
 	sscanf(Tok, "%" SCNd64 "%" SCNd64, &F, &L);
+	#endif
 	if (F == -1 || L == -1) {
 		*Ptr = '-';
 		for (i = 0; i < (int)strlen(Tok); i++) {
@@ -833,8 +881,8 @@ static int CreateHTTPRangeResponseHeader(
 	Ptr = Ptr + 1;
 	if (FileLength < 0) {
 		int ret = HTTP_REQUEST_RANGE_NOT_SATISFIABLE;
-		if ((*Ptr == '0') && (*(Ptr+1) == '-') && (*(Ptr+2) == '\0'))
-		{
+		if ((*Ptr == '0') && (*(Ptr + 1) == '-') &&
+			(*(Ptr + 2) == '\0')) {
 			Instr->IsRangeActive = 0;
 			ret = HTTP_OK;
 		}
@@ -1235,10 +1283,13 @@ static int process_request(
 				goto error_handler;
 			}
 
-			UpnpFileInfo_set_CtrlPtIPAddr(finfo, &info->foreign_sockaddr);
+			UpnpFileInfo_set_CtrlPtIPAddr(
+				finfo, &info->foreign_sockaddr);
 
-			if (httpmsg_find_hdr(req, HDR_USER_AGENT, &hdr_value) != NULL) {
-				UpnpFileInfo_strncpy_Os(finfo, hdr_value.buf, hdr_value.length);
+			if (httpmsg_find_hdr(req, HDR_USER_AGENT, &hdr_value) !=
+				NULL) {
+				UpnpFileInfo_strncpy_Os(
+					finfo, hdr_value.buf, hdr_value.length);
 			}
 
 			/* get file info */
@@ -1340,6 +1391,9 @@ static int process_request(
 		/*          goto error_handler; */
 		/*      } */
 	}
+	RespInstr->CorsHeader = (gWebserverCorsString.length > 0)
+					? gWebserverCorsString.buf
+					: NULL;
 	RespInstr->ReadSendSize = UpnpFileInfo_get_FileLength(finfo);
 	/* Check other header field. */
 	code = CheckOtherHTTPHeaders(
@@ -1378,7 +1432,7 @@ static int process_request(
 			    resp_minor,
 			    "R"
 			    "T"
-			    "GKLD"
+			    "GKLAD"
 			    "s"
 			    "tcS"
 			    "Xc"
@@ -1386,8 +1440,9 @@ static int process_request(
 			    HTTP_PARTIAL_CONTENT, /* status code */
 			    UpnpFileInfo_get_ContentType(
 				    finfo), /* content type */
-			    RespInstr,      /* range info */
-			    RespInstr,      /* language info */
+			    RespInstr,	    /* range info */
+			    RespInstr,	    /* language info */
+			    RespInstr,	    /* Access-Control-Allow-Origin */
 			    "LAST-MODIFIED: ",
 			    &aux_LastModified,
 			    X_USER_AGENT,
@@ -1402,7 +1457,7 @@ static int process_request(
 			    "R"
 			    "N"
 			    "T"
-			    "GLD"
+			    "GLAD"
 			    "s"
 			    "tcS"
 			    "Xc"
@@ -1411,8 +1466,9 @@ static int process_request(
 			    RespInstr->ReadSendSize, /* content length */
 			    UpnpFileInfo_get_ContentType(
 				    finfo), /* content type */
-			    RespInstr,      /* range info */
-			    RespInstr,      /* language info */
+			    RespInstr,	    /* range info */
+			    RespInstr,	    /* language info */
+			    RespInstr,	    /* Access-Control-Allow-Origin */
 			    "LAST-MODIFIED: ",
 			    &aux_LastModified,
 			    X_USER_AGENT,
@@ -1425,7 +1481,7 @@ static int process_request(
 			    resp_major,
 			    resp_minor,
 			    "RK"
-			    "TLD"
+			    "TLAD"
 			    "s"
 			    "tcS"
 			    "Xc"
@@ -1433,7 +1489,8 @@ static int process_request(
 			    HTTP_OK, /* status code */
 			    UpnpFileInfo_get_ContentType(
 				    finfo), /* content type */
-			    RespInstr,      /* language info */
+			    RespInstr,	    /* language info */
+			    RespInstr,	    /* Access-Control-Allow-Origin */
 			    "LAST-MODIFIED: ",
 			    &aux_LastModified,
 			    X_USER_AGENT,
@@ -1448,7 +1505,7 @@ static int process_request(
 				    resp_minor,
 				    "R"
 				    "N"
-				    "TLD"
+				    "TLAD"
 				    "s"
 				    "tcS"
 				    "Xc"
@@ -1458,7 +1515,8 @@ static int process_request(
 					    ->ReadSendSize, /* content length */
 				    UpnpFileInfo_get_ContentType(
 					    finfo), /* content type */
-				    RespInstr,      /* language info */
+				    RespInstr,	    /* language info */
+				    RespInstr, /* Access-Control-Allow-Origin */
 				    "LAST-MODIFIED: ",
 				    &aux_LastModified,
 				    X_USER_AGENT,
@@ -1471,7 +1529,7 @@ static int process_request(
 				    resp_major,
 				    resp_minor,
 				    "R"
-				    "TLD"
+				    "TLAD"
 				    "s"
 				    "tcS"
 				    "Xc"
@@ -1479,7 +1537,8 @@ static int process_request(
 				    HTTP_OK, /* status code */
 				    UpnpFileInfo_get_ContentType(
 					    finfo), /* content type */
-				    RespInstr,      /* language info */
+				    RespInstr,	    /* language info */
+				    RespInstr, /* Access-Control-Allow-Origin */
 				    "LAST-MODIFIED: ",
 				    &aux_LastModified,
 				    X_USER_AGENT,
@@ -1686,9 +1745,8 @@ void web_server_callback(
 	membuffer_init(&headers);
 	membuffer_init(&filename);
 
-	/*Process request should create the different kind of header depending
-	 * on the */
-	/*the type of request. */
+	/* Process request should create the different kind of header depending
+	 * on the type of request. */
 	ret = process_request(
 		info, req, &rtype, &headers, &filename, &xmldoc, &RespInstr);
 	if (ret != HTTP_OK) {
